@@ -6,7 +6,9 @@ import csv
 import logging
 import os
 import re
+import sys
 import tempfile
+from functools import partial
 
 from lxml import etree
 
@@ -21,7 +23,7 @@ from .utils import (
 )
 from .view import View
 
-Manifests = {"__odoo__.py", "__openerp__.py", "__manifest__.py"}
+Manifests = ["__manifest__.py", "__odoo__.py", "__openerp__.py"]
 _logger = logging.getLogger(__name__)
 
 
@@ -77,7 +79,7 @@ class Module:
 
     @property
     def installable(self):
-        return self.manifest.get("installable", "")
+        return self.manifest.get("installable", False)
 
     @property
     def license(self):
@@ -147,15 +149,19 @@ class Module:
         self.hashsum = hexhash_files(files_list, self.path)
 
     def _load_python(self, path, filename):
-        def parse_python(path, filename):
-            name = os.path.join(path, filename)
-            with open(name) as fp:
-                return ast.parse(fp.read())
+        def parse_python(filepath, version=None):
+            with open(filepath) as fp:
+                data = fp.read()
 
-        try:
-            return parse_python(path, filename)
-        except (SyntaxError, TabError):
-            filepath = os.path.join(path, filename)
+            # Python 3.8 allows setting the feature level
+            if version:
+                parsed = ast.parse(data, feature_version=version)
+                _logger.warning("Feature version %s %s", version, filepath)
+                self.status.add("feature-%s-%s" % version)
+                return parsed
+            return ast.parse(data)
+
+        def port_fix_file(filepath):
             with tempfile.NamedTemporaryFile("w+") as tmp:
                 tmp.file.write(open(filepath, "r").read())
                 if try_automatic_port(tmp.name):
@@ -164,7 +170,25 @@ class Module:
                 if fix_indentation(tmp.name):
                     _logger.warning("Fixed indentation %s", filepath)
                     self.status.add("indent-fix")
-                return parse_python(*os.path.split(tmp.name))
+                return parse_python(tmp.name)
+
+        versions = [None]
+        if sys.version_info >= (3, 8):
+            versions.append((3, 6))
+
+        funcs = [partial(parse_python, version=ver) for ver in versions]
+        funcs.append(port_fix_file)
+
+        exc = None
+        filepath = os.path.join(path, filename)
+        for func in funcs:
+            try:
+                return func(filepath)
+            except (SyntaxError, TabError) as e:
+                exc = e
+
+        _logger.error("Not parsable %s: %s", filepath, exc)
+        raise exc
 
     def _parse_class_def(self, obj):
         model = Model.from_ast(obj)
@@ -200,7 +224,8 @@ class Module:
                 if mod != self.name:
                     self.imports.add(mod)
                 continue
-            elif imp.split(".", 1)[0] in ["odoo", "openerp"]:
+
+            if imp.split(".", 1)[0] in ["odoo", "openerp"]:
                 continue
 
             p = path
@@ -264,7 +289,8 @@ class Module:
             view = View.from_xml(self.name, node)
             if not view:
                 continue
-            elif view.name in self.views:
+
+            if view.name in self.views:
                 self.views[view.name].update(view)
             else:
                 self.views[view.name] = view
@@ -315,7 +341,7 @@ class Module:
         module.status = set(data["status"])
         module.language = data["language"]
         module.words = set(data["words"])
-        module.hashsum = set(data["hashsum"])
+        module.hashsum = data["hashsum"]
         return module
 
     @classmethod
@@ -372,7 +398,7 @@ class Module:
         return module
 
     @classmethod
-    def find_modules(cls, paths, depth=None):
+    def find_modules_iter(cls, paths, depth=None):
         result = {}
         if isinstance(paths, str):
             paths = [paths]
@@ -395,7 +421,7 @@ class Module:
             if module is not None:
                 name = module.name
                 if name not in result:
-                    result[name] = module
+                    yield name, module
             else:
                 sub_paths = [
                     os.path.join(path, p)
@@ -404,4 +430,6 @@ class Module:
                 ]
                 paths.extend((p, d + 1) for p in sub_paths if os.path.isdir(p))
 
-        return result
+    @classmethod
+    def find_modules(cls, paths, depth=None):
+        return dict(cls.find_modules_iter(paths, depth))
